/*****************************************************************************\
* (c) Copyright 2000-2020 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef     DETDESC_GEOMETRYINFOEXCEPTION_H
#define     DETDESC_GEOMETRYINFOEXCEPTION_H 1 
/// GaudiKernel
#include "GaudiKernel/GaudiException.h" 
///
class MsgStream;
class IGeometryInfo;
///

/** @class GeometryInfoException GeometryInfoException.h DetDesc/GeometryInfoException.h

    Exception class used in GeometryInfo objects 
    
    @author Vanya Belyaev 


    Change to deal with IGeometryInfo

    @author Juan Palacios

*/

class GeometryInfoException : public GaudiException 
{  
public:
  /// constructor 
  GeometryInfoException( const std::string  & name     , 
                         const IGeometryInfo * gi   = 0 ,
                         const StatusCode   & sc = StatusCode::FAILURE );
  ///
  GeometryInfoException( const std::string    & name      , 
                         const GaudiException & ge        , 
                         const IGeometryInfo   * gi   =  0 ,
                         const StatusCode     & sc = StatusCode::FAILURE ) ;
  /// destructor 
  virtual ~GeometryInfoException() throw();
  ///
  virtual std::ostream& printOut( std::ostream& os = std::cerr ) const ; 
  virtual MsgStream&    printOut( MsgStream&    os             ) const ;
  ///
  virtual GaudiException* clone() const;  
  ///
private:
  ///
  const IGeometryInfo*   m_gie_geometryInfo  ; 
  ///
};
///

#endif  //  DETDESC_GEOMETRYINFOEXCEPTION_H

