/*****************************************************************************\
* (c) Copyright 2000-2020 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/

// GaudiKernel
#include "GaudiKernel/System.h"
// DetDecs 
#include "DetDesc/Surface.h"
#include "DetDesc/TabulatedProperty.h"


////////////////////////////////////////////////////////////////////////////////
Surface::Surface ( const std::string& /*name*/ ) 
  : DataObject   (        )
  , m_model      ()
  , m_finish     ()
  , m_type       () 
  , m_value      ()
  , m_firstVol   () 
  , m_secondVol  ()
  , m_props      ()
{}
////////////////////////////////////////////////////////////////////////////////
Surface::~Surface() { m_props.clear() ;}
////////////////////////////////////////////////////////////////////////////////
std::ostream&     Surface::fillStream ( std::ostream& s ) const 
{
  s << " Surface " 
    << "  model="      << std::setw(1)   << model  () 
    << "  finish="     << std::setw(1)   << finish () 
    << "  type="       << std::setw(1)   << type   () 
    << "  value="      << std::setw(10)  << value  ()
    << "\tfirstVol='"  << firstVol ()    << "'"
    << "\tsecondVol='" << secondVol()    << "'" 
    << "\t#props="     << std::setw(2)   << m_props.size() 
    << std::endl;
  for (Tables::const_iterator it = m_props.begin() ; m_props.end() != it ; ++it)
    { s << "\t" << (*it) ; } 
  return s;
}
////////////////////////////////////////////////////////////////////////////////
MsgStream&        Surface::fillStream ( MsgStream&    s ) const 
{
  s << " Surface " 
    << "  model="      << std::setw(1)  << model  () 
    << "  finish="     << std::setw(1)  << finish () 
    << "  type="       << std::setw(1)  << type   () 
    << "  value="      << std::setw(10) << value  ()
    << "\tfirstVol='"  << firstVol ()   <<"'"
    << "\tsecondVol='" << secondVol()   <<"'" 
    << "\t#props="     << std::setw(2)  << m_props.size() 
    << endmsg ;
  for (Tables::const_iterator it = m_props.begin() ; m_props.end() != it ; ++it)
    { s << "\t" << (*it) ; } 
  return s;
}    
////////////////////////////////////////////////////////////////////////////////








