/*****************************************************************************\
* (c) Copyright 2000-2020 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef _STDetFun_H_
#define _STDetFun_H_

#include <functional>
#include <numeric>
#include "STDet/DeSTBaseElement.h"

namespace STDetFun {

// functors 
  class SortByY
    : public std::binary_function<DeSTBaseElement*,DeSTBaseElement*,bool> 
  {
  public:

    inline bool operator() (DeSTBaseElement* obj1 , DeSTBaseElement* obj2 ) const 
    { 
      return 
        ( !obj1 ) ? true  : 
        ( !obj2 ) ? false : obj1->globalCentre().y() < obj2->globalCentre().y() ; 
    }
    ///
  };


  /// equal by nickname
  template <class TYPE>
  class equal_by_name: public std::unary_function<TYPE, bool>{
    std::string name;
  public:
    explicit equal_by_name(const std::string& testname): name(testname){}
    inline bool operator() (TYPE obj) const{
      return obj->nickname() == name;
    }  
  };

}

#endif // _STDataFunctor_H_
