/*****************************************************************************\
* (c) Copyright 2000-2020 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
// Include files

// from Gaudi
#include "GaudiKernel/AlgFactory.h"

// local
#include "UnpackProtoParticle.h"

//-----------------------------------------------------------------------------
// Implementation file for class : UnpackProtoParticle
//
// 2008-11-14 : Olivier Callot
//-----------------------------------------------------------------------------

// Declaration of the Algorithm Factory
DECLARE_ALGORITHM_FACTORY( UnpackProtoParticle )

//=============================================================================
// Standard constructor, initializes variables
//=============================================================================
  UnpackProtoParticle::UnpackProtoParticle( const std::string& name,
                                            ISvcLocator* pSvcLocator)
    : GaudiAlgorithm ( name , pSvcLocator )
{
  declareProperty( "InputName" , m_inputName  = LHCb::PackedProtoParticleLocation::Charged );
  declareProperty( "OutputName", m_outputName = LHCb::ProtoParticleLocation::Charged );
  declareProperty( "AlwaysCreateOutput", m_alwaysOutput = false );
  //setProperty( "OutputLevel", 1 );
}

//=============================================================================
// Destructor
//=============================================================================
UnpackProtoParticle::~UnpackProtoParticle() {}

//=============================================================================
// Main execution
//=============================================================================
StatusCode UnpackProtoParticle::execute()
{
  if ( msgLevel(MSG::DEBUG) ) debug() << "==> Execute" << endmsg;

  // If input does not exist, and we aren't making the output regardless, just return
  if ( !m_alwaysOutput && !exist<LHCb::PackedProtoParticles>(m_inputName) )
    return StatusCode::SUCCESS;

  const LHCb::PackedProtoParticles* dst =
    getOrCreate<LHCb::PackedProtoParticles,LHCb::PackedProtoParticles>( m_inputName );
  if ( msgLevel(MSG::DEBUG) )
    debug() << "Found " << dst->protos().size() << " PackedProtoParticles at '"
            << m_inputName << "'" << endmsg;

  LHCb::ProtoParticles* newProtoParticles = new LHCb::ProtoParticles();
  put( newProtoParticles, m_outputName );

  // unpack
  const LHCb::ProtoParticlePacker packer(*this);
  packer.unpack( *dst, *newProtoParticles );

  if ( msgLevel(MSG::DEBUG) )
    debug() << "Created " << newProtoParticles->size() << " ProtoParticles at '"
            << m_outputName << "'" << endmsg;

  return StatusCode::SUCCESS;
}

//=============================================================================
