/*****************************************************************************\
* (c) Copyright 2000-2020 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
// ============================================================================
// Include files
// ============================================================================
// AIDA
// ============================================================================
#include "GaudiKernel/Kernel.h"
//-- to avoid that the class macro trick affects them
#include <string>
#include <vector>
/// @FIXME: AIDA interfaces visibility
#define class class GAUDI_API
#include "AIDA/IHistogram1D.h"
#include "AIDA/IHistogram2D.h"
#include "AIDA/IHistogram3D.h"
#include "AIDA/IProfile1D.h"
#include "AIDA/IProfile2D.h"
#undef class
// ============================================================================
// ROOT
// ============================================================================
#include "TH1D.h"
#include "TH2D.h"
#include "TH3D.h"
#include "TProfile.h"
#include "TProfile2D.h"
// ============================================================================
// GaudiKernel
// ============================================================================
#include "GaudiKernel/HistogramBase.h"
// ============================================================================
// GaudiAlg
// ============================================================================
#include "GaudiUtils/Aida2ROOT.h"
// ============================================================================
/** @file
 *  Implementation file for class Gaudi::Utils::Aida2ROOT
 *  @author Vanya BELYAEV ibelyaev@physics.syr.edu
 *  @date  2007-01-23
 */
// ============================================================================
// get the underlying pointer for 1D-histogram
// ============================================================================
TH1D* Gaudi::Utils::Aida2ROOT::aida2root ( AIDA::IHistogram1D* aida )
{
  if ( 0 == aida ) { return 0 ; }
  Gaudi::HistogramBase* base = dynamic_cast<Gaudi::HistogramBase*> ( aida ) ;
  if ( 0 == base ) { return 0 ; }
  return dynamic_cast<TH1D*>( base->representation() ) ;
}
// ============================================================================
// get the underlying pointer for 2D-histogram
// ============================================================================
TH2D* Gaudi::Utils::Aida2ROOT::aida2root ( AIDA::IHistogram2D* aida )
{
  if ( 0 == aida ) { return 0 ; }
  Gaudi::HistogramBase* base = dynamic_cast<Gaudi::HistogramBase*> ( aida ) ;
  if ( 0 == base ) { return 0 ; }
  return dynamic_cast<TH2D*>( base->representation() ) ;
}
// ============================================================================
// get the underlying pointer for 3D-histogram
// ============================================================================
TH3D* Gaudi::Utils::Aida2ROOT::aida2root ( AIDA::IHistogram3D* aida )
{
  if ( 0 == aida ) { return 0 ; }
  Gaudi::HistogramBase* base = dynamic_cast<Gaudi::HistogramBase*> ( aida ) ;
  if ( 0 == base ) { return 0 ; }
  return dynamic_cast<TH3D*>( base->representation() ) ;
}
// ============================================================================
// get the underlying pointer for 1D-profile
// ============================================================================
TProfile* Gaudi::Utils::Aida2ROOT::aida2root ( AIDA::IProfile1D*   aida )
{
  if ( 0 == aida ) { return 0 ; }
  Gaudi::HistogramBase* base = dynamic_cast<Gaudi::HistogramBase*> ( aida ) ;
  if ( 0 == base ) { return 0 ; }
  return dynamic_cast<TProfile*>( base->representation() ) ;
}
// ============================================================================
// get the underlying pointer for 2D-profile
// ============================================================================
TProfile2D* Gaudi::Utils::Aida2ROOT::aida2root ( AIDA::IProfile2D*   aida )
{
  if ( 0 == aida ) { return 0 ; }
  Gaudi::HistogramBase* base = dynamic_cast<Gaudi::HistogramBase*> ( aida ) ;
  if ( 0 == base ) { return 0 ; }
  return dynamic_cast<TProfile2D*>( base->representation() ) ;
}
// ============================================================================
// get root representation for other cases
// ============================================================================
TObject* Gaudi::Utils::Aida2ROOT::aida2root ( AIDA::IHistogram*   aida )
{
  if ( 0 == aida ) { return 0 ; }
  Gaudi::HistogramBase* base = dynamic_cast<Gaudi::HistogramBase*> ( aida ) ;
  if ( 0 == base ) { return 0 ; }
  return base->representation() ;
}
// ============================================================================
// The END
// ============================================================================
