/*****************************************************************************\
* (c) Copyright 2000-2020 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#include "FileReadTool.h"
#include <fstream>

#include "GaudiKernel/ToolFactory.h"
DECLARE_COMPONENT( FileReadTool )

FileReadTool::FileReadTool( const std::string& type,
                            const std::string& name,
                            const IInterface* parent):
  base_class(type, name, parent)
{
  //declareInterface<IFileAccess>(this);
  m_protocols.push_back("file");
}

FileReadTool::~FileReadTool(){}

std::auto_ptr<std::istream> FileReadTool::open(const std::string &url) {
  // remove the optional "file://" from the beginning of the url
  std::string path;
  if ( url.substr(0,7) == "file://" ) {
    path = url.substr(7);
  } else {
    path = url;
  }
  return std::auto_ptr<std::istream>(new std::ifstream(path.c_str()));
}

const std::vector<std::string> &FileReadTool::protocols() const
{
  return m_protocols;
}
