/*****************************************************************************\
* (c) Copyright 2000-2020 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef HLTTRACKREPORTSDECODER_H 
#define HLTTRACKREPORTSDECODER_H 1

// Include files
// from Gaudi
#include "GaudiAlg/GaudiAlgorithm.h"
#include "DAQKernel/DecoderAlgBase.h"

/** @class HltTrackReportsDecoder HltTrackReportsDecoder.h
 *  
 *
 *  @author Sebastian Neubert
 *  @date   2014-01-16
 *
 *  Algorithm to read HltTracks from Raw Data and create containers on TES
 *
 */
class HltTrackReportsDecoder : public Decoder::AlgBase {
public: 

  enum HeaderIDs { kVersionNumber=1 };

  /// Standard constructor
  HltTrackReportsDecoder( const std::string& name, ISvcLocator* pSvcLocator );

  StatusCode initialize() override;    ///< Algorithm initialization
  StatusCode execute   () override;    ///< Algorithm execution

private:

  
  /// location of HltTrackReports 
  StringProperty m_HltTrackReportsLocation;

  /// SourceID to decode. source ids are linked to track stages in TrackNames.trackingSources 
  std::map<std::string,unsigned> m_map;

  unsigned int m_callcount;

};

#endif // HLTTRACKREPORTSDECODER_H
