/*****************************************************************************\
* (c) Copyright 2000-2020 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#include "ConfigTarFileAccessSvc.h"

#include "GaudiKernel/SvcFactory.h"
#include "GaudiKernel/System.h"
#include "tar.h"


using ConfigTarFileAccessSvc_details::TarFile;

// Factory implementation
DECLARE_SERVICE_FACTORY( ConfigTarFileAccessSvc )

//=============================================================================
// Standard constructor, initializes variables
//=============================================================================
ConfigTarFileAccessSvc::ConfigTarFileAccessSvc( const std::string& name,
                                                ISvcLocator* pSvcLocator )
    : ConfigArchiveAccessSvc( name, pSvcLocator ), m_file{ nullptr }
{
    std::string def( System::getEnv( "HLTTCKROOT" ) );
    if ( !def.empty() ) def += "/config.tar";
    declareProperty( "File", m_name = def );
    declareProperty( "Mode", m_mode = "ReadOnly" );
    declareProperty("CompressOnWrite", m_compress = true );
}

//=============================================================================
// Finalization
//=============================================================================
StatusCode ConfigTarFileAccessSvc::finalize()
{
    m_file.reset( nullptr ); // close file if still open
    return ConfigArchiveAccessSvc::finalize();
}


IArchive* ConfigTarFileAccessSvc::file() const
{
    if ( !m_file ) {
        if ( m_mode != "ReadOnly" && m_mode != "ReadWrite" &&
             m_mode != "Truncate" ) {
            error() << "invalid mode: " << m_mode << endmsg;
            return 0;
        }
        std::ios::openmode mode = ( m_mode == "ReadWrite" )
                                 ? ( std::ios::in | std::ios::out | std::ios::ate )
                                 : ( m_mode == "Truncate" )
                                       ? ( std::ios::in | std::ios::out | std::ios::trunc )
                                       : std::ios::in;
        info() << " opening " << m_name << " in mode " << m_mode << endmsg;
        m_file.reset( new TarFile( m_name, mode, m_compress ) );
        if ( !*m_file ) {
            error() << " Failed to open " << m_name << " in mode " << m_mode
                    << endmsg;
            error() << std::string( strerror( errno ) ) << endmsg;
            m_file.reset( nullptr );
        }
    }
    return m_file.get();
}

