/*****************************************************************************\
* (c) Copyright 2000-2020 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef PROCESSPHASE_H
#define PROCESSPHASE_H

// Include files
#include "GaudiAlg/GaudiSequencer.h"

// Typedefs
typedef std::vector<std::string> VectorName;

/** @class ProcessPhase ProcessPhase.h LHCbAlgs/ProcessPhase.h
 *
 * Processing phase of LHCb application
 *
 * Creates and invokes subdetector processing algorithms
 * Convention: algorithm name = \<phase name\> + \<detector name\> 
 *
 * @author: Marco Cattaneo
 * @date:   17th December 1999
 */


class ProcessPhase : public GaudiSequencer {
public:
	ProcessPhase( const std::string& name, ISvcLocator *svcloc ); ///> Constructor
	virtual ~ProcessPhase();                                      ///> Destructor

	StatusCode initialize(); ///> Create and initialise sequences of this phase

private:
  VectorName m_detList;    ///> List of subdetectors to be processed
};

#endif    // PROCESSPHASE_H
