/*****************************************************************************\
* (c) Copyright 2000-2020 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef KERNEL_ILHCBMAGNETSVC_H
#define KERNEL_ILHCBMAGNETSVC_H 1

// Include files
// from STL
#include <string>

// from Gaudi
#include "GaudiKernel/IAlgTool.h"
#include "GaudiKernel/IMagneticFieldSvc.h"

namespace LHCb {
  class MagneticFieldGrid ;
}

static const InterfaceID IID_ILHCbMagnetSvc ( "ILHCbMagnetSvc", 1, 0 );

/** @class ILHCbMagnetSvc ILHCbMagnetSvc.h Kernel/ILHCbMagnetSvc.h
 *  LHCb extensions to the Gaudi IMagneticFieldSvc interface
 *
 *  @author Marco Cattaneo
 *  @date   2008-07-18
 */
class ILHCbMagnetSvc : virtual public IMagneticFieldSvc 
{

public:
  /// Virtual destructor
  virtual ~ILHCbMagnetSvc() {}

  // Return the interface ID
  static const InterfaceID& interfaceID() { return IID_ILHCbMagnetSvc; }

  virtual bool   useRealMap()            const = 0; ///< True if using measured map
  virtual bool   isDown()                const = 0; ///< True if the down polarity map is loaded
  virtual double signedRelativeCurrent() const = 0; ///< scale factor including polarity and current

  /** Implementation of IMagneticFieldSvc interface.
   * @param[in]  xyz Point at which magnetic field vector will be given
   * @param[out] fvec Magnectic field vector.
   * @return StatusCode SUCCESS if calculation was performed.
   */
  virtual StatusCode fieldVector( const ROOT::Math::XYZPoint& xyz,
                                  ROOT::Math::XYZVector& fvec ) const = 0;

  /** Implementation of IMagneticFieldSvc interface.
   * @param[in]  xyz Point at which magnetic field vector will be given
   * @return fvec Magnectic field vector.
   */
  virtual ROOT::Math::XYZVector fieldVector( const ROOT::Math::XYZPoint& xyz ) const = 0;

  /// Get direct access to the field grid
  virtual const LHCb::MagneticFieldGrid* fieldGrid() const = 0 ;

};
#endif // KERNEL_ILHCBMAGNETSVC_H
