/*****************************************************************************\
* (c) Copyright 2000-2020 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef RELATIONS_ISCONVERTIBLE_H 
#define RELATIONS_ISCONVERTIBLE_H 1
// ============================================================================
// Include files
// ============================================================================
// Boost 
// ============================================================================
#include "boost/static_assert.hpp"
// ============================================================================
// use the trick from A.Alexandrescu's book 
// ============================================================================
namespace Relations 
{
  // ==========================================================================
  namespace detail
  {
    // ========================================================================
    struct _Small {                 } ;
    struct _Big   { char dummy[2] ; } ;
    BOOST_STATIC_ASSERT( sizeof(_Small) != sizeof(_Big) )  ;
    // ========================================================================
    template <class T, class U>
    struct Conversion 
    {
      // ======================================================================
      static _Small Test  ( U   ) ;
      static _Big   Test  ( ... ) ;
      static T      MakeT (     ) ;
      // ======================================================================
    public:
      // ======================================================================
      enum { value = sizeof( Test ( MakeT() ) ) == sizeof( _Small ) } ;
      // ======================================================================
    } ;    
    // ========================================================================
  } // end of namespace detail
  // ==========================================================================
} // end of namespace Relations 
// ============================================================================
//#endif 
// ============================================================================
namespace Relations 
{
  // ==========================================================================
  /** @class IsConvertible IsConvertible.h Relations/IsConvertible.h
   *
   *  @todo re-enable boost::is_cinvertible 
   *  @warning temporary disable boost::ins_convertible, 
   *           since it is not compatible with Reflex 
   *           for VC7.1 and gcc4.3
   *
   *  @author Vanya BELYAEV ibelyaev@physics.syr.edu
   *  @date   2006-06-11
   */
  template <class T, class U>
  struct IsConvertible 
  {
    // ========================================================================
    enum 
      {
        same  = false ,
        value = Relations::detail::Conversion<T,U>::value       
      } ;
    // ========================================================================
  } ;
  // ==========================================================================
  template <class T>
  struct IsConvertible<T,T> 
  { enum { same  = true , value = true } ; } ;
  // ==========================================================================
} // end of namespace Relations
// ============================================================================
// The END 
// ============================================================================
#endif // RELATIONS_ISCONVERTIBLE_H
// ============================================================================
