#!/usr/bin/env python
###############################################################################
# (c) Copyright 2000-2020 CERN for the benefit of the LHCb Collaboration      #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
# =============================================================================
# =============================================================================
## @file functions.py LoKiMC/trees.py
#  The set of basic decorator for Tree-objects from LoKiMC library
#  The file is a part of LoKi and Bender projects
#
#        This file is a part of LoKi project - 
#    "C++ ToolKit  for Smart and Friendly Physics Analysis"
#
#  The package has been designed with the kind help from
#  Galina PAKHLOVA and Sergey BARSUK.  Many bright ideas, 
#  contributions and advices from G.Raven, J.van Tilburg, 
#  A.Golutvin, P.Koppenburg have been used in the design.
#
#  @author Vanya BELYAEV ibelyaev@physics.syr.edu
# =============================================================================
"""
The set of basic decorators for Tree-objects from LoKiMC library

      This file is a part of LoKi project - 
'C++ ToolKit  for Smart and Friendly Physics Analysis'

The package has been designed with the kind help from
Galina PAKHLOVA and Sergey BARSUK.  Many bright ideas, 
contributions and advices from G.Raven, J.van Tilburg, 
A.Golutvin, P.Koppenburg have been used in the design.
"""
# =============================================================================
__author__  = "Vanya BELYAEV ibelyaev@physics.syr.edu" 
__date__    = "????-??-??"
# =============================================================================

import LoKiCore.decorators    as      _LoKiCore

# Namespaces:
from LoKiCore.basic import cpp, std, LoKi 
Decays   = cpp.Decays

_MCP = 'const LHCb::MCParticle*'

_decorated = _LoKiCore.decorateTrees (
    ( Decays.iTree_        ( _MCP ) ,
      Decays.Tree_         ( _MCP ) ,
      Decays.Trees.Any_    ( _MCP ) ,
      Decays.Trees.None_   ( _MCP ) ,
      Decays.Trees.Stable_ ( _MCP ) ,
      Decays.Trees.MCExclusive    ,
      Decays.Trees.MCInclusive    ,
      Decays.Trees.MCOptional     ,
      Decays.Trees.Photos         ,
      Decays.Trees.PhotosOptional ) , LoKi.Dicts.TreeOps( _MCP ) )


## decay trees
iMCTree             = Decays.iTree_        ( _MCP )
MCTree              = Decays.Tree_         ( _MCP )
MCAny               = Decays.Trees.Any_    ( _MCP ) () ## instance!!!
MCNone              = Decays.Trees.None_   ( _MCP ) () ## instance!!!
MCStable            = Decays.Trees.Stable_ ( _MCP ) 
MCExclusive         = Decays.Trees.MCExclusive
MCInclusive         = Decays.Trees.MCInclusive
MCOptional          = Decays.Trees.MCOptional
MCPhotos            = Decays.Trees.Photos
MCPhotosOptional    = Decays.Trees.PhotosOptional
Photos              = Decays.Trees.Photos
PhotosOptional      = Decays.Trees.PhotosOptional

IMCDecay = Decays.IMCDecay
MCFinder = _LoKiCore.decorateFinder (
    IMCDecay.Finder               ,
    LoKi.Dicts.FinderDicts_(_MCP)
    ) 

# =============================================================================
if '__main__' == __name__ :
    
    print 80*'*'
    print __doc__
    print ' Author  : ' , __author__
    print ' Date    : ' , __date__    
    print 80*'*'
    for i in dir() : print i 
    
# =============================================================================
# The END 
# =============================================================================
